function [sOut, warn] = makelegalname(sIn)
% S = MAKELEGALNAME(S)
% 	Converts strings S into names that are legal for both MATLAB
% 	variables and .MAT files. Input S may be a character array or
% 	a cell array of strings
% 	
% 	* Non-alphanumeric characters are replace by '_'.
% 	* Names which start in a non-alphabetic character are
% 	  prefixed by 'n'. 
% 	* Names are chopped to ensure a maximum length of 27 (hence,
% 	  filenames are no longer than 31 characters once the '.MAT'
% 	  extension has been added).
% 	  
% 	If the resulting list of names contains fewer unique names
% 	than the input, then a warning is issued. This checking
% 	process is case-insensitive, to warn against potential
% 	overwriting of files on a case-insensitive file system.
% 	The warning can be suppressed by "catching" it in a second
% 	output argument, as follows:
% 		[S MSG] = MAKELEGALNAMES(S);
% 		% then issue ERROR(MSG)	if it matters, or ignore if not.

% Part of PFCMP version 2.5.41, an extension to the PSIGNIFIT toolbox for MATLAB versions 5 and up.
% The license and NO WARRANTY statement for PSIGNIFIT also applies to this release - see psych_legal.m
% Copyright (c) J.Hill 2002.
% mailto:psignifit@bootstrap-software.org
% http://www.bootstrap-software.org/psignifit/

if iscellstr(sIn)
	siz = size(sIn);
	sIn = char(sIn(:));
elseif isstr(sIn) & ndims(sIn) < 3
	siz = [];
else
	error('input must be a string matrix or a cell array of strings')
end

sOut = sIn;
% sOut(ismember(sOut), '.') = [];
illegalChars = ~isletter(sOut) & ~ismember(sOut, '0123456789_');
trailingSpace = fliplr(cumprod(fliplr(isspace(sOut)), 2));
sOut(illegalChars & ~trailingSpace) = '_';

sOut = fliplr(sOut);
sOut(:, end+1) = ' ';
sOut(~isletter(sOut(:, end-1)), end) = 'n';
sOut = deblank(strjust(fliplr(sOut), 'left'));

maxLen = 27;  % max length of a filename is 31 and we should allow for '.mat'
sOut(:, maxLen+1:end) = [];

warn = '';
nDistinctIn = size(unique(lower(sIn), 'rows'), 1);
nDistinctOut = size(unique(lower(sOut), 'rows'), 1);
if nDistinctOut < nDistinctIn, warn = 'some names are duplicated once they have been made legal'; end
if nargout < 2, warning(warn), end

if ~isempty(siz), sOut = reshape(cellstr(sOut), siz); end
