function [sFull, sSlim] = pfcmp(d1, d2, varargin)
% s = pfcmp(d1, d2, ...)
% 
% Comparison of psychometric function fits to data sets d1 and d2.
% The PSIGNIFIT engine is used in order to fit a function to d1, to d2, and to d1 and d2 combined.
% The observed differences between thresholds (t2-t1) and slopes (s2-s1) are noted.
% The engine then performs Monte Carlo simulations of d1 and of d2 under the assumption that they were
% both generated by binomial processes from the /same/ function, i.e. from the combined fit.
% The result is a Monte Carlo distribution of threshold differences and slope differences, against which
% the observed difference (t2-t1, s2-s1) is compared using two-dimensional density estimation with a
% gaussian kernel. A density value is computed for each point of the distribution, and for the observed
% value. The significance value p is then given by an estimate of the probability that densities from the
% Monte Carlo distribution are smaller than that from the observed difference.
% 
% Additional (optional) input arguments to PFCMP are options of the form expected by PFIT, for
% specifying the function shape used for fitting, the number of intervals in a forced choice design, the
% number of simulations to run, etc. Further, parameters may be fixed specifically for the combined fit
% (see below).
% 
% If an output argument is requested, a structure is returned. Field names have the following meanings:
%        one:  fits to data d1
%        two:  fits to data set d2
%   combined:  fits to the combined data [d1; d2]
%       diff:  differences between estimates obtained from d2 and d1
%         pa:  parameters [alpha beta gamma lambda]
%         ts:  threshold and slope [t s]
%        gof:  goodness-of-fit statistics (for combined fit and [non-refitted] simulated fits w.r.t each data set)
%          L:  likelihood
%        est:  values estimated from the data
%        sim:  values from simulation
%        val:  estimated or simulated values
%       dens:  density estimate corresponding to each row of val, computed from the corresponding sim.val distribution
%          p:  significance probability based on comparison of each dens value against the whole sim.dens distribution
% 
% The critical result is therefore diff.ts.est.p
% If no output arguments are requested, a graphical representation of the test is plotted.
% Note that, as density estimation is being carried out on two-dimensional measures, a larger number of simulations is
% required than might be expected in most one-dimensional contexts, in order for the value of p to be reasonably repeatable
% from one call to PFCMP to the next. The default here is 1999 but I suggest at least 4999.
% 
% Note that the test is not as such a bootstrap test (requiring correction for bootstrap error) - rather, it is a direct
% Monte Carlo test of a hypothesis H_0, namely that the two data sets both came from the same particular function,
% viz the maximum-likelihood fit to the combined data. As such, the accuracy of the test is only dependent on the
% number of simulations. However, note that the /scope/ of the test is limited:  the maximum-likelihood fit to the
% combined data is only /one/ possible common function from which d1 and d2 might have come. To rule out the null-
% hypothesis, ALL possible common functions must be tested, which is of course impossible. However, it is often
% useful to try a number of different functions. For the purposes of such tests, it is possible to specify certain
% additional options, beyond those supported by PFIT:
% 
%    COMBINED_LAMBDA :    fixes lambda_AB, the lambda value for the combined fit
%    COMBINED_GAMMA  :    fixes gamma_AB, the gamma value for the combined fit
%    COMBINED_PARAMS :    bypasses the combined fit, instead using a fixed set of parameters [alpha beta gamma lambda]
% 
% In particular, it is often the case that p increases if one simply assumes that the underlying common function has
% a higher lambda (manipulated using COMBINED_LAMBDA). This is unfortunate, because lambda is so hard to estimate
% accurately. A plot of p against lambda_AB is, however, instructive, particularly in conjunction with a plot of the
% relative likelihoods of lambda_AB, because it allows the reader to judge, based on the profile likelihood combined.L,
% and also on his or own prior expectation of what values lambda might take, the most reasonable and conservative value
% for p.
% 
% Likelihood of different H_0 curves may also be assessed by the goodness-of-fit tests:
%	cpe(s.one.gof.est, s.one.gof.sim)
%	cpe(s.two.gof.est, s.two.gof.sim)

% Part of PFCMP version 2.5.41, an extension to the PSIGNIFIT toolbox for MATLAB versions 5 and up.
% The license and NO WARRANTY statement for PSIGNIFIT also applies to this release - see psych_legal.m
% Copyright (c) J.Hill 2002.
% mailto:psignifit@bootstrap-software.org
% http://www.bootstrap-software.org/psignifit/


if nargin < 1, d1 = []; end
if nargin < 2, d2 = []; end

if nargin <= 2 & isstr(d1)
	if any(strcmpi(d1, {'plotsetup', 'plotfits', 'plotsims', 'ebars', 'doplots'}))
		if isstr(d2), d2 = evalin('caller', d2, '[]'); end
		if isempty(d2)
			[out err] = eval(d1);
		else
			if ~isstruct(d2) | prod(size(d2)) ~= 1, error('second argument should be a 1-by-1 struct'), end
			[out err] = eval([d1 '(d2)']);
		end
		error(err)
		if nargout, s = out; end
		return
	end
end
plotnow = (nargout == 0);

if isstr(d1), varargin = [{d1 d2} varargin];  d1 = []; d2 = [];
elseif isstr(d2), varargin = [{d2} varargin];  d2 = [];
end

if isempty(d1) & isempty(d2) 
	[d1 d2] = nicedata;
	swap = getprefs('swap', 0);
	if swap
		d2(:, 1) = d2(:, 1) - 1.3;
		[d2 d1] = deal(d1, d2);
	end
end

[x1 y1 n1] = parsedataset(d1);
[x2 y2 n2] = parsedataset(d2);
s.one.dat = [x1 y1 n1];
s.two.dat = [x2 y2 n2];
s.combined.dat = [s.one.dat; s.two.dat];
s.diff = [];

userOpts = varargin;
defaultOpts = {
	'shape'			'c'
	'n_intervals'	2
	'verbose'		0
	'cuts'			0.5
	'runs'			1999
}';

s.opts = batch(defaultOpts{:}, userOpts{:});
combinedGamma = getopt(s.opts, 'combined_gamma', []);
combinedLambda = getopt(s.opts, 'combined_lambda', []);
combinedParams = getopt(s.opts, 'combined_params', []);
s.opts = batch(s.opts, 'combined_gamma', [], 'combined_lambda', [], 'combined_params', []);
shape = getopt(s.opts, 'shape', 'logistic');
cut = getopt(s.opts, 'cuts', [0.2 0.5 0.8]); if length(cut) ~= 1, error('CUTS must be scalar here'), end

opts1 = {
	'runs'          0
	'write_pa_est'  's.one.pa.est'
	'write_th_est'  's.one.ts.est(1)'
	'write_sl_est'  's.one.ts.est(2)'
}';
opts2 = {
	'runs'          0
	'write_pa_est'  's.two.pa.est'
	'write_th_est'  's.two.ts.est(1)'
	'write_sl_est'  's.two.ts.est(2)'
}';

psignifit(s.one.dat, batch(s.opts, opts1{:}));
psignifit(s.two.dat, batch(s.opts, opts2{:}));

extraCombinedOpts = {};
if ~isempty(combinedGamma), extraCombinedOpts = [extraCombinedOpts {'fix_gamma', combinedGamma}]; end
if ~isempty(combinedLambda), extraCombinedOpts = [extraCombinedOpts {'fix_lambda', combinedLambda}]; end
if ~isempty(combinedParams)
	s.combined.pa.est = combinedParams;
else
	s.combined.pa.est = psignifit(s.combined.dat, batch(s.opts, 'runs', 0, extraCombinedOpts{:}));
end
[s.combined.ts.est(2) s.combined.ts.est(1)] = findslope(shape, s.combined.pa.est, cut);
p = psi(shape, s.combined.pa.est, s.combined.dat(:, 1));
n = s.combined.dat(:, 3);
y = s.combined.dat(:, 2);
r = round(s.combined.dat(:, 2) .* n);
s.combined.L = prod(nchk(n, r) .* (p.^r) .* ((1-p).^(n-r)));

if plotnow
	ax = plotsetup(s);
	axes(ax(1))
	plotfits(s);
	axes(ax(2)), cla
	drawnow
end

opts1 = {
	'gen_params'          s.combined.pa.est
	'data_x'              x1
	'data_y'              y1
	'data_n'              n1
	'write_pa_sim'        's.one.pa.sim'
	'write_th_sim'        's.one.ts.sim(:, 1)'
	'write_sl_sim'        's.one.ts.sim(:, 2)'
	'write_st_est'	      's.one.gof.est'
	'write_st_sim'        's.one.gof.sim'
	'write_random_seed'   's.one.randomSeed'
}';
opts2 = {
	'gen_params'	      s.combined.pa.est
	'data_x'              x2
	'data_y'              y2
	'data_n'              n2
	'write_pa_sim'        's.two.pa.sim'
	'write_th_sim'        's.two.ts.sim(:, 1)'
	'write_sl_sim'        's.two.ts.sim(:, 2)'
	'write_st_est'	      's.two.gof.est'
	'write_st_sim'        's.two.gof.sim'
	'write_random_seed'   's.two.randomSeed'
}';

psignifit(batch(s.opts, opts1{:}))
psignifit(batch(s.opts, opts2{:}))

s.one.gof.est(:, 3) = []; % kd correlation is no help in judging combined fit
s.one.gof.sim(:, 3) = []; %
s.two.gof.est(:, 3) = []; %
s.two.gof.sim(:, 3) = []; %

s.diff.ts.est.val = s.two.ts.est - s.one.ts.est;
s.diff.ts.sim.val = s.two.ts.sim - s.one.ts.sim;

[s.diff.ts.sim.dens, s.diff.ts.sim.p, s.diff.ts.est.dens, s.diff.ts.est.p] = ...
	densnd(s.diff.ts.sim.val, [], [], s.diff.ts.est.val);

s.diff.ts.sim.p = 1 - s.diff.ts.sim.p;
s.diff.ts.est.p = 1 - s.diff.ts.est.p;

if plotnow
	axes(ax(2))
	plotsims(s);
	hold on
	ebars(s);
	hold off
	drawnow
end

sFull = s;
fn = flattenstruct(s);
sSlim = removefields(s, grep(fn, '.sim'));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [out, err] = plotfits(s)

out = []; err = '';
if nargin < 1, err = 'no input'; return, end

shape = getopt(s.opts, 'shape', 'logistic');
out = gca;

datProps = {'marker', 'o'};
lineProps = {'linestyle', '-', 'linewidth', 1};
combinedProps = {'linestyle', '--', 'linewidth', 2, 'color', [0 0.8 0]};
colours = {[0 0 1]; [1 0 0]};

hold off
d1 = plotpd(s.one.dat, datProps{:}, 'color', colours{1});
hold on
d2 = plotpd(s.two.dat, datProps{:}, 'color', colours{2});
markercolour([d1;d2], 0.6, 0.2)
w1 = getwarnstate; warning off
plotpf(shape, s.combined.pa.est, combinedProps{:})
plotpf(shape, s.one.pa.est, lineProps{:}, 'color', colours{1})
plotpf(shape, s.two.pa.est, lineProps{:}, 'color', colours{2})
setwarnstate(w1)
sendtofront([d1;d2])
hold off


shape = strrep(psychf(shape), 'Gaussian', 'normal'); shape(1) = upper(shape(1));
txt = texprintf('%s function%\n%\n\lambda_{AB} = %.3f', shape, s.combined.pa.est(4));
alt = texprintf('%s function%\n%\n$\lambda_{\mbox{\tiny AB}} = %.3f$', shape, s.combined.pa.est(4));
if getopt(s.opts, 'n_intervals', 2) == 1
	txt = texprintf('%s%\n%\n\gamma_{AB} = %.3f', txt, s.combined.pa.est(3));
	alt = texprintf('%s%\n%\n$\gamma_{\mbox{\tiny AB}} = %.3f$', alt, s.combined.pa.est(3));
end
latext(0.95, 0.05, txt, alt, 'fontweight', 'bold', 'units', 'normalized', ...
	'horizontalalignment', 'right', 'verticalalignment', 'bottom')

latext('-xlabel', 'stimulus', 'stimulus', 'fontweight', 'bold')
latext('-ylabel', 'performance', 'performance', 'fontweight', 'bold')
xtick auto, ytick auto, tickfmt(gca)
fakelabel(gca, 'x', 25, 'fontweight', 'bold')
fakelabel(gca, 'y', 40, 'fontweight', 'bold')
fakelabel(gca, 't', 10, 'fontweight', 'bold')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [out, err] = plotsims(s)

out = []; err = '';
if nargin < 1, err = 'no input'; return, end
sim = eval('s.diff.ts.sim', '[]');
if isempty(sim), err = 'input struct contains no simulations'; return, end

out = gca;
colormap(flipud(jet(256)))
cla, set(gca, 'visible', 'on')
xl = sensiblerange(s.diff.ts.sim.val(:, 1), [-4 4])';
yl = sensiblerange(s.diff.ts.sim.val(:, 2), [-4 4])';
xlim([-1 1] * max(abs([xl 1.25 * s.diff.ts.est.val(1)])))
ylim([-1 1] * max(abs([yl 1.25 * s.diff.ts.est.val(2)])))
hold on
hSim = patchplot(s.diff.ts.sim.val, s.diff.ts.sim.p, 'size', 2);
xlabel 'threshold difference', ylabel 'slope difference'
pbaspect auto
cb = colorbar;
pbaspect(cb, 'auto')
set(cb, 'ylim', [0 1])
pt = get(cb, 'children');
set(pt, 'ydata', [0 1]);
ti = get(cb, 'title');
set(ti, 'string', 'p')
set(allchild(cb), 'deletefcn', '')
if getprefs('fake', 0)
	prefs = getprefs('fakeaxescontent');
	setprefs('fakeaxescontent', 'oversample', 1)
	fakeaxescontent([gca cb])
	setprefs('fakeaxescontent', prefs)
end

props = {'linewidth',  2, 'linestyle', '--', 'color', [0.65 0.65 0.65]};
hObs = [line([1 1] * s.diff.ts.est.val(1), ylim, props{:}); line(xlim, [1 1] * s.diff.ts.est.val(2), props{:})];
hObs = dotplot(s.diff.ts.est.val, s.diff.ts.est.p, 'marker', 'o', 'markersize', 16, 'linewidth', 2, 'markeredgecolor', [0.65 0.65 0.65]);
% set(hObs, {'markeredgecolor'}, contrastingcolour(get(hObs, {'markerfacecolor'})))



hold off
grid on

xtick auto, ytick auto, tickfmt(gca)
ytick(cb, 'auto'), tickfmt(cb, '%.1f')

latext('-title', texprintf('p = %.3f', s.diff.ts.est.p), texprintf('$p = %.3f$', s.diff.ts.est.p))
fakelabel([gca cb], 'x', 25, 'fontweight', 'bold')
fakelabel([gca cb], 'y', 40, 'fontweight', 'bold')
fakelabel([gca cb], 't', 10, 'fontweight', 'bold')


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [out, err] = plotsetup(s)

out = []; err = '';
figar = [1 1.4];
scrsiz = measure(0, 'pixels', 'screensize');
fig = get(0, 'currentfigure');
siz = 0.9 * scrsiz(4) * figar / figar(2);
maxw = scrsiz(3) * 0.95;
if siz(1) > maxw, siz = siz * maxw/siz(1); end
offset = (scrsiz(3:4) - siz) ./ [2 2.5];

if isempty(fig)
	fig = figure('resize', 'off', 'units', 'pixels', 'position', [offset siz]);
	set(fig, 'units', 'pixels', 'position', [offset siz])
	set(fig, 'units', 'normalized')
else
	resizebox(fig, 'pixels', siz)
end

siz = measure(fig, 'pixels', 'position'); siz = siz(3:4);
edges = [55 15 20 45];

axsiz = siz .* [1 0.5] - [sum(edges([1 2])) sum(edges([3 4]))];
axsiz = [edges([1 4]) axsiz];
tags = {'pfcmp1', 'pfcmp2'};
ax1 = findobj(fig, 'type', 'axes', 'tag', tags{1});
delete(ax1(2:end)), ax1(2:end) = [];
if isempty(ax1)
	ax1 = axes('tag', tags{1}, 'units', 'pixels', 'position', axsiz + [0 siz(2)/2 0 0]);
	set(ax1, 'plotboxaspectratiomode', 'auto', 'units', 'normalized')
end

ax2 = findobj(fig, 'type', 'axes', 'tag', tags{2});
delete(ax2(2:end)), ax2(2:end) = [];
if isempty(ax2)
	ax2 = axes('tag', tags{2}, 'units', 'pixels', 'position', axsiz);
	set(ax2, 'plotboxaspectratiomode', 'auto', 'units', 'normalized')
end
out = [ax1;ax2];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [out, err] = doplots(s)

out = []; err = '';
if nargin < 1, err = 'no input'; return, end
sim = eval('s.diff.ts.sim', '[]');
if isempty(sim), err = 'input struct contains no simulations'; return, end

ax = plotsetup(s);
set(ax(2), 'visible', 'off')
axes(ax(1))
plotfits(s);
axes(ax(2))
plotsims(s);
hold on
ebars(s);
hold off
drawnow


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [out, err] = ebars(s)

out = []; err = '';
if nargin < 1, err = 'no input'; return, end
sim = eval('s.diff.ts.sim', '[]');
if isempty(sim), err = 'input struct contains no simulations'; return, end

x = min(xlim) + 0.05 * diff(xlim);
y = min(ylim) + 0.05 * diff(ylim);
props = {'color', [0.65 0.65 0.65], 'linewidth', 2, 'markersize', 12, 'marker', 'none'};
h{1} = psycherrbar(s.diff.ts.est.val(1), quantile(s.diff.ts.sim.val(:, 1), phi([-2 -1 1 2])'), y, 'h', props{:});
h{2} = psycherrbar(s.diff.ts.est.val(2), quantile(s.diff.ts.sim.val(:, 2), phi([-2 -1 1 2])'), x, 'v', props{:});
out = h;


